import os
from ctypes import cdll, c_char_p


def decrypt_passwd(passwd, so_path="./lib/passwd.so"):
    try:
        lib = cdll.LoadLibrary(so_path)
        lib.passedDecrypt.argtypes = [c_char_p]
        lib.passedDecrypt.restype = c_char_p
        password = lib.passedDecrypt(passwd.encode()).decode()
        return password
    except Exception:
        return passwd


class BasicConfig(object):
    basedir = os.path.abspath(os.path.dirname(__file__))

    SQLALCHEMY_TRACK_MODIFICATIONS = False
    TEMPLATES_AUTO_RELOAD = True

    # Security
    SESSION_COOKIE_HTTPONLY = True
    REMEMBER_COOKIE_HTTPONLY = True
    REMEMBER_COOKIE_DURATION = 3600

    # SQLALCHEMY_ENGINE_OPTIONS
    SQLALCHEMY_ENGINE_OPTIONS = {
        'pool_pre_ping': True,   # 每次從 Connection Pool 取得連線時，就試著執行一次相當於 SELECT 1 的 SQL ，如果有問題，就可以重新建立新的連線取代失效的連線。
        'pool_recycle': 60,      # 设置 DBAPI connection 存活多久断开
        'pool_timeout': 100,     # 从 pool 中获取新的连接等待时间，一般是指等待 pool 中连接可用的时间
        'pool_size': 10,         # The size of the pool to be maintained, defaults to 5.
        'max_overflow': 100,     # 允许超过 pool_size 多少
    }




from hashlib import sha256
from .exception import TokenMissingError, MacValidateError


def encrypt_token(token: str, timestamp: str):
    return sha256((token+timestamp).encode("utf-8")).hexdigest()


def mac_validation(req: dict, token: str):
    '''Mac Validate'''
    mac, timestamp = req.get("mac"), req.get("timestamp")
    if mac is None:
        raise TokenMissingError('0103', 'MAC token invalid/missing.')
    elif timestamp is None:
        raise TokenMissingError('0104', 'Timestamp token invalid/missing.')
    else:
        if mac != encrypt_token(token, timestamp):
            raise MacValidateError('9903', 'Mac token validate failed.')


def get_policy_score(total_score: float):
    return round((total_score*(-2) + 1), 6)




import numpy as np
import pandas as pd
from sqlalchemy import update
from sqlalchemy.pool import QueuePool
from sqlalchemy import asc, desc, create_engine
from sqlalchemy.orm import sessionmaker, scoped_session, load_only


def get_orm_profile(host='192.168.10.201', db='acqfd_test', **kwargs):

    profile = {
        'host': host,
        'port': '3306',
        'user': 'acqfd',
        'pwd': 'acqfd16313302',
        'db': db
    }

    if kwargs:
        profile.update(kwargs)

    return profile


class Orm():
    def __init__(
        self,
        profile=get_orm_profile(),
        pool=False,
        pool_size=250,
        max_overflow=0,
        pool_recycle=10,
        pool_timeout=10,
        logger=None
    ):
        self.profile = profile
        self.DB_URL = f"{profile['user']}:{profile['pwd']}@{profile['host']}:{profile['port']}"
        self.sql_connect = f"mysql+pymysql://{self.DB_URL}/{profile['db']}?charset=utf8mb4&binary_prefix=true"
        if pool:
            self.engine = create_engine(
                self.sql_connect,
                pool_size=pool_size,
                max_overflow=max_overflow,
                pool_recycle=pool_recycle,
                pool_timeout=pool_timeout,
                pool_pre_ping=True,
                poolclass=QueuePool,
                pool_use_lifo=True
            )
        else:
            self.engine = create_engine(self.sql_connect, echo=False)
        self._sessionmaker = sessionmaker(bind=self.engine)
        self.session = scoped_session(self._sessionmaker)
        self.logger = logger

    def create_schema(self):
        schemaName = self.profile['db']
        engine = create_engine(f'mysql+pymysql://{self.DB_URL}')

        conn = engine.connect()
        if schemaName.lower() not in conn.dialect.get_schema_names(conn):
            if self.logger is not None:
                self.logger.warning(
                    f'Schema {schemaName} not exist, create {schemaName}.')
            engine.execute(f"CREATE SCHEMA {schemaName}")
            conn.close()
            if self.logger is not None:
                self.logger.warning(f'Done creating schema {schemaName}')

        # disconnect database
        engine.dispose()

    def get_session(self):
        return scoped_session(self._sessionmaker)

    @staticmethod
    def close_connection(session):
        session.close()

    def create_table(self, base, table):
        '''Create a table in DB'''

        engine = create_engine(self.sql_connect, echo=True)
        table
        base.metadata.create_all(engine)

    def truncate_table(self, table_name: str):
        con = self.engine.connect()
        con.execute(f"TRUNCATE TABLE {table_name};")
        con.close()
        print("TRUNCATE has completed")

    def delete(self, data: pd.DataFrame, tablename):
        '''Delete data in DB'''

        session = self.get_session()
        for i in range(data.shape[0]):
            kwargs = data.iloc[i].to_dict()
            query_data = session.query(tablename).filter_by(**kwargs).all()
            for row in query_data:
                session.delete(row)
                session.commit()
        session.close()

    def get_log_position(self):
        result = pd.read_sql(f"show master status;", self.session.bind)
        position = result['Position'].values[0]
        return position

    def check_sn(self, tablecolumn, sn: str):
        '''Check if a sn is in a table'''

        session = self.get_session()
        try:
            check_sn = session.query(tablecolumn).filter_by(sn=sn).all()
            session.close()

            if len(check_sn) == 0:
                return True
            return False

        except:
            session.rollback()

    def query(self, table, *filterBy, **conditions):
        '''Get data from DB'''

        session = self.get_session()
        query = session.query(table).filter(*filterBy)

        if conditions.get('orderBy'):
            if conditions['orderBy'] == 'asc':
                query = query.order_by(asc(conditions['orderTarget']))
            else:
                query = query.order_by(desc(conditions['orderTarget']))

        if conditions.get('limit'):
            query = query.limit(conditions['limit'])

        if conditions.get('fields'):
            query = query.options(load_only(*conditions['fields']))

        result = pd.read_sql_query(query.statement, session.connection())
        session.close()

        for c in ['pk_id', 'create_time']:
            if c in result.columns:
                result = result.drop(c, axis=1)
        return result

    def query_filter(self, tablename, limit: int = None, order_by: tuple = None, fields: list = None, *args):
        '''Get data from DB'''

        session = self.get_session()
        query = session.query(tablename).filter(*args)

        if order_by:
            if order_by[1] == 'asc':
                query = query.order_by(asc(order_by[0]))
            else:
                query = query.order_by(desc(order_by[0]))

        if limit:
            query = query.limit(limit)

        if fields:
            query = query.options(load_only(*fields))

        result = pd.read_sql(query.statement, session.bind)
        session.close()

        return result

    def elk_query_filter(self, tablename, limit=None, *args):
        query = self.session.query(tablename).filter(*args)
        if limit:
            query = query.limit(limit)
        return pd.read_sql(query.statement, self.session.bind)

    def update(self, table, update_content: dict, *filterBy):
        '''Update data in table'''

        session = self.get_session()
        session.execute(
            update(table).where(*filterBy).values(update_content)
        )
        session.commit()
        session.close()

    def delete(self, table, *args):
        '''Delete data in table'''

        session = self.get_session()
        query_data = session.query(table).filter(*args).all()
        for row in query_data:
            session.delete(row)
            session.commit()
        session.close()

    def check_exist(self, tablename, **kwargs):
        '''用於確認指定資料是否存在'''
        session = self.get_session()
        q = session.query(tablename).filter_by(**kwargs)
        check_result = session.query(q.exists()).scalar()
        session.close()
        return check_result

    def add_data(self, table, **input_data):
        '''Add single data row to table'''

        try:
            session = self.get_session()
            data = table(**input_data)
            session.add(data)
            session.commit()
        except:
            session.rollback()
        finally:
            session.close()

    def data_to_DB(self, df: pd.DataFrame, table):
        '''Import dataframe to DB'''

        # 轉換時間格式
        for col in df.columns:
            if df[col].dtype == pd._libs.tslibs.timestamps.Timestamp:
                df[col] = df[col].astype(str).apply(
                    lambda x: None if x == 'NaT' else x)

        # 補空值
        if df.isnull().sum().sum():
            df = df.where(pd.notnull(df), '')

        datarows = np.array(df.to_dict('records'))
        alldata = np.array([table(**row) for row in datarows])

        # upload data
        session = self.get_session()
        session.add_all(alldata)
        session.commit()
        session.close()


def main():
    from sqlalchemy import Column, text, Integer
    from sqlalchemy.dialects.mysql import TIMESTAMP
    from sqlalchemy.ext.declarative import declarative_base

    orm = Orm(
        profile={
            'host': '192.168.10.201',
            'port': '3306',
            'user': 'acqfd',
            'pwd': 'acqfd16313302',
            'db': 'acqfd_test'
        }
    )

    Base = declarative_base()
    metadata = Base.metadata

    class TestTable(Base):
        __tablename__ = 'udid_history'

        pk_id = Column(Integer, primary_key=True,
                       autoincrement=True, unique=True)
        create_time = Column(TIMESTAMP(fsp=6), nullable=False,
                             server_default=text("CURRENT_TIMESTAMP(6)"))

    # 建立資料表
    orm.create_table(Base, TestTable)
    print('Create table done.')


if __name__ == "__main__":
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Time:2020.09.12
# @author:xhrg
# @email:634789257@qq.com
import logging
import os

import urllib.request
from urllib.error import HTTPError
from urllib import parse


def http_request(url, timeout, headers={}):
    try:
        request = urllib.request.Request(url, headers=headers)
        res = urllib.request.urlopen(request, timeout=timeout)
        body = res.read().decode("utf-8")
        return res.code, body
    except HTTPError as e:
        if e.code == 304:
            logging.getLogger(__name__).warning(
                "http_request error,code is 304, maybe you should check secret")
            return 304, None
        logging.getLogger(__name__).warning(
            f"http_request error,code is {e.code}, msg is {e.msg}")
        raise e


def url_encode(params):
    return parse.urlencode(params)


def makedirs_wrapper(path):
    os.makedirs(path, exist_ok=True)





class ModelHandler:
    def __init__(self, db, table_info, table_result, logger):
        self.db = db
        self.info = table_info
        self.result = table_result
        self.logger = logger

    def save_data(self, data_type='info', **input_data):
        try:
            if data_type == 'info':
                input_model = self.info(**input_data)
            else:
                input_model = self.result(**input_data)

            self.db.session.add(input_model)
            self.db.session.commit()
            return input_model

        except:
            self.db.session.rollback()
            self.logger.exception(
                f"[{input_model.serial_number}]Save data into {input_model.__tablename__} failed")



import requests
from datetime import datetime
from healthcheck import HealthCheck

from ..api import encrypt_token


class DataPayload:
    Payloads = {
        'card-abnormal': {
            "diia_serial_number": "test_diia_serial_number", 
            "connector_id": "test_connector_id", 
            "institute_id": "test_institute_id", 
            "operator_id": "test_operator_id", 
            "serial_number": "unittestSN12345786789", 
            "customer_servertime": "2021-11-09 05:10:10", 
            "pan_hash": "unit_test_card_0", 
            "merchant_id": "23456", 
            "purchase_amount": "2013000", 
            "purchase_currency": "901", 
            "udid": "unit_test_udid_0", 
            "ip_country": "Taiwan", 
            "os_name": "Windows", 
            "browser_language": "zh-TW", 
            "ip_request": "192.168.100.203"
        },
        'card-testing': {
            "diia_serial_number": "test_diia_serial_number", 
            "institute_id": "I999",
            "operator_id": "O001",
            "connector_id": "8909191002",
            "serial_number": "unittestSN12345786789", 
            "udid": "1234567890123456789",
            "customer_servertime": "2021-11-20 12:59:56",
            "pan_hash": "34m4MOQAZM9/BQVpSimvaLjeYgBUxqBi6zr4SGLqg9QbJ1D82zQxtvu4YnoyA==1",
            "pan_expire": "2032",
            "merchant_id": "451615MID001",
            "card_bin": "101671****6012",
            "ip_request": "201.167.110.111",
            "ip_source": "201.167.117.111"
        },
        'fraud-detect': {
            'udid': '1516109167423655936',
            'serial_number': 'daa07c27302eb4eb5aa05e93b63b812a159e651frJhNcdcf1n74TwAS',
            'diia_serial_number': 'c695f0aae9e528f22a38e201d2a9c8e5',
            'connector_id': 'AI_CONNECTOR',
            'operator_id': 'SYSOPERATOR',
            'institute_id': '2020',
            'client_account': 'admin',
            'hardware_device_type': 'Desktop or Laptop',
            'hardware_device_model': 'Macintosh',
            'hardware_device_brand': 'Apple',
            'hardware_pointing_method': 'Mouse',
            'hardware_gpu_name': 'Intel(R) Iris(TM) Graphics 6100',
            'hardware_cpu_architecture': 'x86-64',
            'os_name': 'macOS',
            'os_platform': 'MacIntel',
            'os_screen_color_depth': '30-bit',
            'os_screen_pixel_ratio': '2',
            'os_screen_resolution_width': '1440',
            'os_screen_resolution_height': '764',
            'os_screen_orientation': 'Landscape',
            'os_local_timezone_offset': '+8',
            'os_local_datetime': 'Mon Nov 22 2021 18:12:00 GMT+0800 (Taipei Standard Time)',
            'os_local_timezone': 'Asia/Taipei',
            'browser_name': 'Safari Browser',
            'browser_engine': 'Webkit',
            'browser_is_tor': 'False',
            'browser_is_brave': 'False',
            'browser_resolution_width': '1440',
            'browser_resolution_height': '764',
            'browser_is_private_mode': 'False',
            'browser_is_use_adblock': 'True',
            'browser_language': 'en',
            'browser_account_login': 'undefined',
            'browser_useragent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36',
            'browser_req_useragent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36',
            'ip_is_proxy': 'False',
            'ip_is_tor': 'False',
            'ip_is_vpn': 'False',
            'ip_country': 'Taiwan',
            'ip_cloud_server': 'False',
            'ip_timezone': 'Asia/Taipei',
            'ip_source': '59.125.100.235',
            'ip_request': '59.125.100.235',
            'ip_isp': 'HiNet',
            'robot': {
                'serial_number': 'wnj31g38qn07u332gwpz8jmxg49zooha7gyfk3w1',
                'client_account': 'admin',
                'platform': 'MacIntel',
                'vendor': 'Google Inc.',
                'renderer': 'Google SwiftShader',
                'useragent': 'Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/95.0.4638.69 Safari/537.36',
                'webdriver': '0',
                'is_private_mode': 'False',
                'eval': 33,
                'mouse_enter': '1',
                'mouse_click': '1',
                'touch_start': '0',
                'tab': '0',
                'paste': '0',
                'is_change_useragent': 'False',
                'is_change_plugins': 'False',
                'is_change_languages': 'False',
                'is_change_webdriver': 'False',
                'is_change_eval': 'False',
                'is_change_platform': 'False',
                'page_time': 10.646,
                'mouse_log': '[{"pageX":439,"pageY":402,"count":1,"timeStamp":"2220.800"},{"pageX":660,"pageY":450,"count":14,"timeStamp":"5034.100"},{"pageX":490,"pageY":714,"count":1,"timeStamp":"5132.300"},{"pageX":466,"pageY":733,"count":1,"timeStamp":"5230.800"},{"pageX":428,"pageY":743,"count":1,"timeStamp":"5329.100"},{"pageX":365,"pageY":735,"count":1,"timeStamp":"5435.700"},{"pageX":327,"pageY":691,"count":1,"timeStamp":"5534.400"},{"pageX":320,"pageY":683,"count":1,"timeStamp":"5624.600"},{"pageX":319,"pageY":682,"count":1,"timeStamp":"5731.100"}]',
                'events_log': '[{"event":"mouseenter","time":1655271109666,"pageX":439,"pageY":402,"name":"inputUsername","input_order":"inputOrder1","timeStamp":2138.5},{"event":"click","time":1655271110179,"pageX":439,"pageY":402,"name":"inputUsername","input_order":"inputOrder1","timeStamp":2645.6}]',
                'resolution_width': '1440',
                'resolution_height': '900',
                'pixel_ratio': '1',
                'source_ip': '102.125.100.235',
                'CreateTime': '2022-06-15T05:31:53.8637Z'
            },
            'deviceinfo_create_time': '2021-11-17 09:50:06'
        },
        'account-takeover': {
            'serial_number': 'test1',
            'diia_serial_number': '123',
            'connector_id': '123',
            'institute_id': '11001',
            'operator_id': '123',
            'account': {'account': 'test11@gmail.com'},
            'device': {
                'udid': '14421239600917281',
                'hardware_device_type': 'Desktop or Laptop',
                'os_name': 'macOS',
                'os_version': '10.15.7'
            },
            'ip': {
                'ip_request': '1250.117.43.39',
                'ip_latitude': '253.0127',
                'ip_longitude': '1121.4609',
                'ip_is_proxy': 'False',
                'ip_is_vpn': 'False',
                'ip_is_tor': 'False',
                'ip_cloud_server': 'False'
            }
        },
        'merchant-risk': {
            'serial_number': 'test',
            'diia_serial_number': '123',
            'connector_id': '123',
            'institute_id': '123',
            'operator_id': '123',
            'client_account': 'admin',
            'merchant_id': 'merchant_3',
            'pan_hash': 'c1014',
            'purchase_amount': 35000000,
            'purchase_currency': 'TWD',
            'merchatrisk_create_time': '2022-08-15 10:10:10'
        },
        'gateway-acqfd': {
            'cifNumber': None,
            'cardScheme': 'M',
            'processType': '01',
            'txInfo': {
                'threeDSRequestorAuthenticationInd': None,
                'threeDSRequestorAuthenticationInfo': {
                    'threeDSReqAuthData': '111',
                    'threeDSReqAuthMethod': None,
                    'threeDSReqAuthTimestamp': None
                },
                'acctType': '01',
                'acquirerBIN': '1231234',
                'acquirerMerchantId': '8909191',
                'addrMatch': None,
                'acctInfo': {
                    'chAccAgeInd': '01',
                    'chAccChange': None,
                    'chAccChangeInd': None,
                    'chAccDate': None,
                    'chAccPwChange': None,
                    'chAccPwChangeInd': None,
                    'nbPurchaseAccount': None,
                    'provisionAttemptsDay': None,
                    'txnActivityDay': None,
                    'txnActivityYear': None,
                    'paymentAccAge': None,
                    'paymentAccInd': None,
                    'shipAddressUsage': None,
                    'shipAddressUsageInd': None,
                    'shipNameIndicator': None,
                    'suspiciousAccActivity': None
                },
                'cardNo': None,
                'cardBin': None,
                'acctID': 'F123456789',
                'transType': None,
                'billAddrCity': None,
                'billAddrCountry': None,
                'billAddrLine1': None,
                'billAddrLine2': None,
                'billAddrLine3': None,
                'billAddrPostCode': None,
                'billAddrState': None,
                'email': None,
                'emailMask': None,
                'homePhone': None,
                'mobilePhone': None,
                'workPhone': None,
                'cardholderName': None,
                'cardExpiryDate': '2512',
                'shipAddrCity': None,
                'shipAddrCountry': None,
                'shipAddrLine1': None,
                'shipAddrLine2': None,
                'shipAddrLine3': None,
                'shipAddrPostCode': None,
                'shipAddrState': None,
                'deviceChannel': '03',
                'purchaseInstalData': None,
                'mcc': None,
                'merchantCountryCode': None,
                'merchantName': None,
                'merchantRiskIndicator': None,
                'messageCategory': '01',
                'purchaseAmount': '99900',
                'purchaseCurrency': '901',
                'purchaseExponent': '2',
                'purchaseDate': None,
                'recurringExpiry': None,
                'recurringFrequency': None,
                'whiteListStatus': None,
                'whiteListStatusSource': None,
                'threeDSRequestorPriorAuthenticationInfo': None,
                'cardholderInfo': None,
                'threeRIInd': None,
                'threeDSRequestorDecMaxTime': None,
                'threeDSRequestorDecReqInd': None,
                'threeDSCompInd': None,
                'threeDSReqAuthMethodInd': None,
                'threeDSRequestorChallengeInd': None,
                'threeDSRequestorID': None,
                'threeDSRequestorName': None,
                'threeDSServerRefNumber': None,
                'browserAcceptHeader': None,
                'browserIP': None,
                'browserJavaEnabled': None,
                'browserJavascriptEnabled': None,
                'browserLanguage': None,
                'browserColorDepth': None,
                'browserScreenHeight': None,
                'browserScreenWidth': None,
                'browserTZ': None,
                'browserUserAgent': None,
                'messageExtension': None,
                'acctNumber': 'xxxxxxxxxxxxxxx',
                'acctNumberMask': '353352******0123',
                'cifNumber': None,
                'payTokenInd': None,
                'payTokenSource': None,
                'messageVersion': None,
                'sdkAppID': None,
                'sdkMaxTimeout': None,
                'threeDSServerTransID': None,
                'dsTransID': None,
                'sdkReferenceNumber': None,
                'realAmount': None,
                'realAmountInUsd': None,
                'mcScore': None,
                'mcDecision': None,
                'mcReasonCode1': None,
                'mcReasonCode2': None
            },
            'diiaInfo': {
                'deviceInfo': None,
                'robotInfo': None,
                'serialNumber': 'sn0000000011',
                'diiaSerialNumber': 'b49448532cd2037af4291f4e8b141a49'
            },
            'ak': None,
            'mode': '1',
            'promoteInfo': None,
            'operatorId': 'HT158001',
            'instituteId': 'acqfdtest',
            'merchantId': None,
            'connectorId': 'IB_TW007_TEST_1',
            'version': '2.0.0',
            'veriIdTransID': 'a17f9339-bafa-456a-917b-4deeb553f955',
            'mac': 'PSf+JCK98+T7wEKo59AupSifzt1gqx+Yuu2y4xv2gFbVB1gmVjBovvEEtPM7T4/0Mc96Kb8uCFEc0uIqOEolXg==',
            'timestamp': '20230309062402',
            'clientTransId': '5681830f-a645-4b20-84be-cf6e3c0f3799'
        },
        'gateway-ibfd': {
            'diia_serial_number': '27e2727bdd52dfdd9f99cab95bdbce07',
            'serial_number': 'TEST-b52aee47514309ce9bfb678d5d56b738bb1a8d5336ETV2LCK5EFJUSN1',
            'loginType': 'account',
            'loginSuccess': 'Y',
            'accountId': 'user02',
            'operatorId': 'HT158001',
            'instituteId': 'ibfdtest',
            'connectorId': 'IB_TW007_TEST_1',
            'version': '2.0.0',
            'mac': 'Q9qmvMsmEKFmh3oJKbAddkcugqP69PuiKgHt4Sa1ff76DGJnnigdXZQADaK+hvXfpi+WmCJ4YHSBaz4M8PH1mg==',
            'timestamp': '1652171287345',
            'firstStepSuccess': 'Y',
            'isStepUp': 'Y'
        },
        'gateway-isrfd': {
            'cifNumber': None,
            'cardScheme': 'V',
            'processType': None,
            'txInfo': {
                'threeDSRequestorAuthenticationInd': '01',
                'threeDSRequestorAuthenticationInfo': {
                    'threeDSReqAuthData': None,
                    'threeDSReqAuthMethod': None,
                    'threeDSReqAuthTimestamp': None
                },
                'acctType': '01',
                'acquirerBIN': '1231234',
                'acquirerMerchantId': '8909191',
                'addrMatch': None,
                'acctInfo': {
                    'chAccAgeInd': '',
                    'chAccChange': '',
                    'chAccChangeInd': '',
                    'chAccDate': '',
                    'chAccPwChange': '',
                    'chAccPwChangeInd': '',
                    'nbPurchaseAccount': '',
                    'provisionAttemptsDay': '',
                    'txnActivityDay': '',
                    'txnActivityYear': '',
                    'paymentAccAge': '',
                    'paymentAccInd': '',
                    'shipAddressUsage': '',
                    'shipAddressUsageInd': '',
                    'shipNameIndicator': '',
                    'suspiciousAccActivity': ''
                },
                'cardNo': None,
                'cardBin': 'demo',
                'acctID': 'Q9X0hNlhnPh8+gA4TVZScqeOO6txgIGjDMSIYcCiunfaYq48F7ee/FmNBPSBtdhaZ1i5xU7d7z7TX5NEqnDSfg==',
                'transType': None,
                'billAddrCity': '',
                'billAddrCountry': '',
                'billAddrLine1': '',
                'billAddrLine2': '',
                'billAddrLine3': '',
                'billAddrPostCode': '',
                'billAddrState': '',
                'email': '',
                'emailMask': '',
                'homePhone': {'cc': '', 'subscriber': '', 'subscriberMask': None},
                'mobilePhone': {'cc': '', 'subscriber': '', 'subscriberMask': None},
                'workPhone': {'cc': '', 'subscriber': '', 'subscriberMask': None},
                'cardholderName': '',
                'cardExpiryDate': '2512',
                'shipAddrCity': '',
                'shipAddrCountry': '',
                'shipAddrLine1': '',
                'shipAddrLine2': '',
                'shipAddrLine3': '',
                'shipAddrPostCode': '',
                'shipAddrState': '',
                'deviceChannel': '02',
                'purchaseInstalData': None,
                'mcc': '5661',
                'merchantCountryCode': '158',
                'merchantName': 'HiTRUST ACQFD Merchant Simulator',
                'merchantRiskIndicator': {
                    'deliveryEmailAddress': '',
                    'deliveryTimeframe': '',
                    'giftCardAmount': '',
                    'giftCardCount': '',
                    'giftCardCurr': '',
                    'preOrderDate': '',
                    'preOrderPurchaseInd': '',
                    'reorderItemsInd': '',
                    'shipIndicator': ''
                },
                'messageCategory': '01',
                'purchaseAmount': '100',
                'purchaseCurrency': '901',
                'purchaseExponent': '2',
                'purchaseDate': '20230912065656',
                'recurringExpiry': '',
                'recurringFrequency': '',
                'whiteListStatus': None,
                'whiteListStatusSource': None,
                'threeDSRequestorPriorAuthenticationInfo': None,
                'cardholderInfo': None,
                'threeRIInd': None,
                'threeDSRequestorDecMaxTime': None,
                'threeDSRequestorDecReqInd': None,
                'threeDSCompInd': 'Y',
                'threeDSReqAuthMethodInd': None,
                'threeDSRequestorChallengeInd': None,
                'threeDSRequestorID': '12128301823081230123',
                'threeDSRequestorName': 'demo',
                'threeDSServerRefNumber': 'demo',
                'browserAcceptHeader': 'demo',
                'browserIP': None,
                'browserJavaEnabled': 'false',
                'browserJavascriptEnabled': 'false',
                'browserLanguage': 'demo',
                'browserColorDepth': 'demo',
                'browserScreenHeight': 'demo',
                'browserScreenWidth': 'demo',
                'browserTZ': None,
                'browserUserAgent': 'demo',
                'messageExtension': None,
                'acctNumber': 'y74bWBsLzBv2Kw2dfRTE3mg8zm4w1Go7nYVpNV1Zjmg/UoepjHTlTq1srdwnrak5GV+w0RulMVtoxotUupS7Ew==',
                'acctNumberMask': '51535200****0123',
                'cifNumber': None,
                'payTokenInd': None,
                'payTokenSource': None,
                'messageVersion': 'demo',
                'sdkAppID': None,
                'sdkMaxTimeout': None,
                'threeDSServerTransID': None,
                'dsTransID': None,
                'sdkReferenceNumber': None,
                'realAmount': None,
                'realAmountInUsd': None,
                'mcScore': None,
                'mcDecision': None,
                'mcReasonCode1': None,
                'mcReasonCode2': None
            },
            'diiaInfo': {
                'deviceInfo': ' ',
                'robotInfo': {
                    'serial_number': 'testZjEwOWQ1MDgtYzdmMy00ZmUwLWFjZDQtODE3NzllMjJlYjM3',
                    'diia_serial_number': 'b9a83708fef10870d0c5973981387298',
                    'client_account': 'ISR_TW008_TEST_1-141',
                    'platform': 'Win32',
                    'vendor': 'Google Inc. (Intel)',
                    'renderer': 'ANGLE (Intel, Intel(R) UHD Graphics 620 Direct3D11 vs_5_0 ps_5_0, D3D11)',
                    'webdriver': '0',
                    'is_private_mode': 'False',
                    'useragent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/116.0.0.0 Safari/537.36',
                    'eval': 33,
                    'is_change_useragent': 'False',
                    'is_change_plugins': 'False',
                    'is_change_languages': 'False',
                    'is_change_webdriver': 'False',
                    'is_change_eval': 'False',
                    'is_change_platform': 'False',
                    'mouse_enter': '1',
                    'mouse_click': '0',
                    'touch_start': '0',
                    'paste': '0',
                    'tab': '0',
                    'page_time': 2.504,
                    'mouse_log': '[{"pageX":1207,"pageY":368,"count":1,"timeStamp":"806.600"},{"pageX":1225,"pageY":374,"count":1,"timeStamp":"847.300"},{"pageX":1228,"pageY":374,"count":1,"timeStamp":"967.300"},{"pageX":1236,"pageY":373,"count":6,"timeStamp":"1607.500"},{"pageX":1240,"pageY":372,"count":1,"timeStamp":"1623.600"},{"pageX":1241,"pageY":372,"count":2,"timeStamp":"1912.200"},{"pageX":1043,"pageY":753,"count":1,"timeStamp":"2006.600"},{"pageX":884,"pageY":936,"count":1,"timeStamp":"2103.700"},{"pageX":836,"pageY":916,"count":1,"timeStamp":"2207.200"},{"pageX":758,"pageY":892,"count":1,"timeStamp":"2305.300"},{"pageX":745,"pageY":894,"count":1,"timeStamp":"2342.200"}]',
                    'events_log': '[{"event":"mouseenter","time":1694501850884,"pageX":948,"pageY":362,"name":"txInfo.purchaseDate","input_order":"inputOrder3","timeStamp":717.8}]',
                    'resolution_width': '1920',
                    'resolution_height': '1080',
                    'pixel_ratio': '0.8999999761581421',
                    'source_ip': '59.125.100.237'
                },
                'serialNumber': 'testZjEwOWQ1MDgtYzdmMy00ZmUwLWFjZDQtODE3NzllMjJlYjM3',
                'diiaSerialNumber': 'b9a83708fef10870d0c5973981387298'
            },
            'operatorId': 'BA036001',
            'instituteId': '11003',
            'merchantId': None,
            'connectorId': 'ISR_AU001_BANO_1',
            'version': '1.0.0',
            'mac': 'nRwtlNRj9FbvYmFcNg9M107nOkXvy8xM+hDkxRgX4GssGNKyySnTJ8XxUGVF7thSd08IbE+qMxcYiRSzt51uyQ==',
            'timestamp': '20230912065656',
            'clientTransId': 'f109d508-c7f3-4fe0-acd4-81779e22eb37'
        }
    }

    def add_payload(self, model_name: str, payload: dict):
        self.Payloads.update({model_name: payload})

    def get(self, model_name: str, token: str, timestamp: str):
        if model_name not in self.Payloads:
            return {}
        
        now = datetime.now()
        payload = self.Payloads.get(model_name)
        
        if 'gateway' in model_name:
            return payload
        
        payload.update({
            "diia_serial_number": f"TEST-{now}",
            "serial_number": f"TEST-{now}",
        })
        return {
            "data_version": "1.0.0",
            "mac": encrypt_token(token, timestamp),
            "timestamp": timestamp,
            "payload": payload
        }


class Healthchecker(DataPayload):
    def __init__(self, model_name: str, token: str, timestamp: str):
        self.model_name = model_name
        self.TOKEN = token
        self.TIMESTAMP = timestamp

    def post_v2(self):
        url = f"http://127.0.0.1:8080/model/predict/{self.model_name}/v2/api/predict"
        input_data = self.get(
            self.model_name, self.TOKEN, timestamp=self.TIMESTAMP)
        
        contents = requests.post(url, json=input_data)
        if contents.status_code == 200:
            output = contents.json()
            if output['return_code'] == "4003":
                return True, "4003"
            return False, output["return_code"]
        return False, str(contents.status_code)
    
    @staticmethod
    def add_healthcheck_url(app, check_func, url: str):
        health_check = HealthCheck()
        health_check.add_check(check_func)
        app.add_url_rule(url, "healthcheck", view_func=lambda: health_check.run())
        return app


from flask import Flask
from opentelemetry import trace
from opentelemetry import metrics
from opentelemetry.exporter.otlp.proto.grpc.trace_exporter import OTLPSpanExporter
from opentelemetry.instrumentation.flask import FlaskInstrumentor
from opentelemetry.sdk.resources import SERVICE_NAME, Resource
from opentelemetry.sdk.trace import TracerProvider
from opentelemetry.sdk.trace.export import BatchSpanProcessor
from opentelemetry.metrics import get_meter

def configure_trace(service_name, service_host, app: Flask):
    trace.set_tracer_provider(
        TracerProvider(resource=Resource.create({SERVICE_NAME: service_name}))
    )

    trace.get_tracer_provider().add_span_processor(
        BatchSpanProcessor(
            OTLPSpanExporter(
                endpoint=service_host
            )
        )
    )

    # RedisInstrumentor().instrument()
    FlaskInstrumentor().instrument_app(app)